require 'active_model'
require 'set'

module Xpost
  class Address
    include ActiveModel::Model

    REQUIRED_ADDRESS_PARAMETERS = Set[:name, :line_1, :city, :state, :postal_code, :country]
    OPTIONAL_ADDRESS_PARAMETERS = Set[:company, :title, :email, :phone_number, :mobile_number, :fax_number,
                                      :district, :line_2, :remarks]

    attr_accessor :name, :line_1, :city, :state, :postal_code, :country
    attr_accessor :company, :title, :email, :phone_number, :mobile_number, :fax_number, :district, :line_2, :remarks

    validates_presence_of :name, :line_1, :city, :state, :postal_code, :country

    def initialize(options = {})
      # (?) Mr., Mrs., Ms., etc.
      @title          = options[:title]
      @name           = options[:name]
      @email          = options[:email]
      @phone_number   = options[:phone_number]
      @mobile_number  = options[:mobile_number]
      @fax_number     = options[:fax_number]
      @company        = options[:company]

      # Address Line 1
      @line_1         = options[:line_1]
      # Address Line 1
      @line_2         = options[:line_2]
      # City
      @city           = options[:city]
      # District (NCR's Four Districts) (Optional)
      @district       = options[:district]
      # State / Province
      @state          = options[:state]
      # Postal Code / ZIP Code
      @postal_code    = options[:postal_code]
      # Country
      @country        = options[:country]

      @remarks        = options[:remarks]
    end

    def attributes
      {
        title: self.title,
        name: self.name,
        email: self.email,
        phone_number: self.phone_number,
        mobile_number: self.mobile_number,
        fax_number: self.fax_number,
        company: self.company,
        line_1: self.line_1,
        line_2: self.line_2,
        city: self.city,
        district: self.district,
        state: self.state,
        postal_code: self.postal_code,
        country: self.country,
        remarks: self.remarks
      }
    end

    def self.sample_address
      address = Xpost::Address.new(
        name: "Bobby Axelrod",
        line_1: "EDSA Shangri-La",
        city: "Mandaluyong",
        state: "Metro Manila",
        postal_code: "1650",
        country: "PH"
      )
      address
    end
  end
end