require 'base64'
require 'openssl'
require 'json'

module Xpost
  module Authentication
    def self.authenticate(api_key, secret_key)
      payload   = payload(api_key)
      signature = signature(header, payload, secret_key)
      token     = "#{header}.#{payload}.#{signature}"
    end

    def self.sample_auth_token
      config = Xpost::Configuration.sample_configuration
      Xpost::Authentication.authenticate(config.api_key, config.secret_key)
    end

    private

    def self.header
      url_safe({ "alg" => "HS256", "typ" => "JWT"}.to_json)
    end

    def self.payload(api_key)
      iat = jti = Time.now.to_i
      url_safe({"sub": api_key, "iat": iat, "jti": jti }.to_json)
    end

    def self.signature(header, payload, secret_key, alg = 'HS256')
      data = "#{header}.#{payload}"
      url_safe(OpenSSL::HMAC.digest('SHA256', secret_key, data))
    end

    def self.url_safe(data)
      Base64.urlsafe_encode64(data).delete('=')
    end
  end
end