require 'active_model'
require 'set'

module Xpost
  class Item
    include ActiveModel::Model

    ITEM_TYPES = Set[:product, :shipping, :tax, :fee, :insurance, :discount]

    attr_accessor :product_type, :description, :amount, :quantity, :metadata

    validates_presence_of :product_type, :description, :amount
    validate :check_item_type

    def initialize(options = {})
      # Product Type
      @product_type = options.key?(:product_type) ? options[:product_type] : "product"
      # Product Description
      @description  = options[:description]
      # (Single) Product Amount
      @amount       = options[:amount]
      # Product Quantity
      @quantity     = options.key?(:quantity) ? options[:quantity].to_i : 1
      # Additional Product Data (For third-party purposes)
      @metadata     = options[:metadata]
    end

    def check_item_type
      if self.product_type.present? && !ITEM_TYPES.include?(self.product_type.to_sym)
        errors.add(:type, "wrong item type")
      end
    end
    
    def attributes
      {
        type: self.product_type,
        description: self.description,
        amount: self.amount,
        quantity: self.quantity,
        metadata: self.metadata
      }
    end
  end
end